/* Copyright (c) 2007 by Tensilica Inc.  ALL RIGHTS RESERVED.
 *  * These coded instructions, statements, and computer programs are the
 *   * copyrighted works and confidential proprietary information of Tensilica Inc.
 *    * They may not be modified, copied, reproduced, distributed, or disclosed to
 *     * third parties in any manner, medium, or form, in whole or in part, without
 *      * the prior written consent of Tensilica Inc.
 *       */

/*
 *  This is simple example flash programmer for the XT-AV60 board.
 *   By default it writes an image from offset 0x61000000 in system RAM to Flash.
 *    It erases the entire Flash (in a single operation) before programming.
 *     It does not perform full Flash status checks or use all its capabilities.
 *      Meant to run under a debugger. The user should connect to the board and load
 *       this program, then load the image, optionally change some variables, and run.
 *        Variables image_base, image_size, flash_base and flash_erase may be changed.
 *         */

#include    <stdlib.h>
#include    <xtensa/xtav60.h>
#include    <xtensa/xtbsp.h>
#include "serial_pl01x.h"
/* Flash device is an Atmel AT49BV322A, ST M29W320xB, or similar. */
/* This program uses chip erase mode so is agnostic to block geometry. */

#define FLASHADDR  0x80000000 //XTBOARD_FLASH_VADDR     /* base address of flash */
#define FLASHSIZE XTBOARD_FLASH_MAXSIZE   /* size of entire flash (bytes) */
#define FLASHTOP  (FLASHADDR + FLASHSIZE) /* top+1 address of entire flash */
#ifdef XTBOARD_RAM_VADDR
#define BINARYADDR (XTBOARD_RAM_VADDR + 0x1000000)  /* base of RAM image */
#else
#define BINARYADDR 0                      /* (must set image_base manually) */
#endif
#define BLOCK_SIZE 0x20000
#define CMDADDR(cmd) ((((cmd)&0xfff)<<1) | FLASHADDR)
#define flashread(addr) (*(volatile short *)addr)
#define flashwrite(addr, data) *(volatile short *)addr = data

/* User can change these variables from debugger after loading. */
/* Set flash_base to locate image other than at base of flash. */
/* Set image_size to avoid programming entire region. */
/* This program erases the entire flash at once, but clearing flash_erase
 *    allows multiple images to be programmed to different flash regions. */
/* To erase entire flash w/o programming, set image_size = 0. */
unsigned flash_base = FLASHADDR;    /* where image starts in flash */
unsigned image_base = BINARYADDR;   /* where image starts in memory */
unsigned image_size = FLASHSIZE;    /* size of image, amount programmed */
int      flash_erase = 1;           /* erase flash before programming */


/* Write string to primary console device. Much lower overhead than printf(). */
static void putstring(const char *s)
{
	char c;

	while ((c = *s++) != '\0') {
		serial_puts(c);
	}
}

/* Write hex number of len digits to primary console device (with leading 0s). */
static void puthex(unsigned hex, unsigned len)
{
	unsigned lsh, mask, nib;

	putstring("0x");
	if (len == 0) return;
	lsh = (len-1) << 2;
	mask = 0xF << lsh;
	while (len-- > 0) {
		nib = (hex & mask) >> lsh;
		serial_puts(nib<10 ? '0'+nib : 'A'+(nib-10));
		hex <<= 4;
	}
}

/* Write message with address and flash status code and exit with error code. */
static void error(int err, const char *msg, unsigned addr, unsigned short stat)
{
	lftprintf("Flash Error !!!");
	flashwrite(FLASHADDR, 0xF0);          /* restore read mode and clear status */
	putstring("Error ");
	putstring(msg); 
	putstring(" at address ");
	puthex(addr, 8);
	putstring(", flash status ");
	puthex(stat, 2);
	putstring("\n");
	//exit(err);
}

/* Erase entire flash. */
static void eraseFlash(void)
{
	flashwrite(CMDADDR(0x555), 0xAA);     /* 6 cycle chip erase command */
	flashwrite(CMDADDR(0xAAA), 0x55);
	flashwrite(CMDADDR(0x555), 0x80);
	flashwrite(CMDADDR(0x555), 0xAA);
	flashwrite(CMDADDR(0xAAA), 0x55);
	flashwrite(CMDADDR(0x555), 0x10);
	while (flashread(FLASHADDR) != 0xFFFFFFFF);
}

/* Erase entire flash. */
static void eraseFlashBlk(int blkno)
{
	unsigned int  BlockStartAddr = blkno * BLOCK_SIZE;
	unsigned short *WrAddr = (unsigned short*)(FLASHADDR+BlockStartAddr);
//	lftprintf("BlockStartAddr 0x%08X BLOCK_SIZE 0x%08X bklno 0x%08X Status Reg 0x%08X \n" ,*(WrAddr+BlockStartAddr),BLOCK_SIZE,blkno,WrAddr[0x28]);
	WrAddr[0x555] = 0x00AA;     /* 6 cycle chip erase command */
	WrAddr[0x2AA] = 0x0055;
	WrAddr[0x555] = 0x0080;
	WrAddr[0x555] = 0x00AA;
	WrAddr[0x2AA] = 0x0055;
	WrAddr[0] = 0x30;     
	while((WrAddr[0] & 0xFFFF) != 0xFFFF);
	lftprintf("Erase Done %08X  \n",WrAddr[0]);

}


int ReadFlash(unsigned short image,unsigned int blkno, unsigned int size)
{
	int lv;
        int counter = 0 ;
	unsigned short iData;
	unsigned int BlockStartAddr = blkno * BLOCK_SIZE;
	unsigned short *ReadAddr = (unsigned short*)(FLASHADDR+BlockStartAddr);
	for(lv = 0 ; lv < size; lv++)
	{
		if(ReadAddr[lv] != 0xAA55){
			lftprintf("ERROR: READADDR 0x%08X DATA 0x%08X\n",&ReadAddr[lv],ReadAddr[lv]);
			counter++;
		}
	}
	lftprintf("BLOCK 0x%08X READ DONE ADDR 0x%08X DATA 0x%08X \n",blkno,&ReadAddr[lv-1],ReadAddr[lv-1]);
     return counter;
}


/* Program image from RAM to flash (caller ensures blank space is available). */
static void programFlash(unsigned short image, unsigned blkno, unsigned size)
{
	unsigned short stat, data7;
	unsigned short *imagepos;
	unsigned int flashpos;

	unsigned int BlockStartAddr = blkno * BLOCK_SIZE;
	int lv ;
	volatile unsigned short *dataAddr;
	unsigned short iData;
	unsigned short  *WrAddr = (unsigned short*)(FLASHADDR+BlockStartAddr);
	image = 0xAA55;
	WrAddr[0] = 0xF0;
	for(lv = 0 ; lv < size ; lv++)
	{
		data7 = image & 0x80;
		WrAddr[0x555] = 0xAA;
		WrAddr[0x2AA] = 0x55;
		WrAddr[0x555] = 0xA0;
		WrAddr[lv] = image;
	
		if(((WrAddr[0x28] & 0x0001) == 0x0001) || ((WrAddr[0x28] & 0x0002) == 0x0002) || ((WrAddr[0x28] & 0x0004) == 0x0004)){
                	lftprintf("STATUS REG %08X WRITE ADDR %08X  \n",WrAddr[0x28],&WrAddr[lv]); 
		}
		while (((stat = WrAddr[lv]) & 0x80) != data7 ) {  /* done? */
			if ((stat & 0x20) == 0) continue; /* no error signal in status bit 5 */
			if (((stat = WrAddr[lv]) & 0x80) == data7)break; /* recheck */
			lftprintf("programming flash word %08X %08X\n", WrAddr[lv],stat);
		}
	}
	lftprintf("Block %08X Completed BlockStartAddr %08X Last byte %08X\n",blkno,&WrAddr[lv-1],WrAddr[lv-1]);
}

void ReadConf(){
 
	unsigned short  *WrAddr = (unsigned short*)FLASHADDR;
        lftprintf("WRSETUP REG %08X \n",WrAddr[0x044]);
        WrAddr[0x044] = 0xA00A0000;
        lftprintf(" AFTER WRSETUP REG %08X \n",WrAddr[0x044]);
        lftprintf("  WRCYC REG %08X \n",WrAddr[0x048]);
        WrAddr[0x048] = 0x0000000E;
        lftprintf(" AFTER  WRCYC REG %08X \n",WrAddr[0x048]);
        lftprintf(" BSYCYC REG %08X \n",WrAddr[0x04C]);
}

/* Erase and program flash. Notify user of progress via display and console. */
int main()
{
	int block_no = 3;
	int data = 0xAA55;
        int counter = 0 ; 
        int err_cnt = 0 ;
	int lv;
	serial_init();
	
	//ReadConf();

	for(lv = 190 ; lv < 1024 ; lv++)
        {
		lftprintf("Block No %d \n",lv);
		eraseFlashBlk(lv);
		programFlash(data,lv,BLOCK_SIZE);
		lftprintf("Program Flash done...\n");
                err_cnt = ReadFlash(data,lv,BLOCK_SIZE);
		lftprintf("Error cnt %08X \n",err_cnt);
	}
	lftprintf("Flash Done !!!\n");

	return 0;
}

